<?php defined('BASEPATH') OR exit('No direct script access allowed');

class Estimates extends Application_controller {
	
	function __construct() {
		
		parent::__construct();
		initial_data_contoller($this);

		if(! (logged_user()->isOwner() || !logged_user()->isMember()) ) redirect('dashboard');
				
	}
	
	public function index() {

		if(logged_user()->isOwner()) {
			$estimates = $this->Estimates->getAll();
		} else {
			$estimates = $this->Estimates->getByClientOnly(logged_user());
		}
		
		tpl_assign('estimates', $estimates);
	
	}

	public function view($access_key) {

		$this->setLayout("page_dialog");
				
		$estimate = $this->Estimates->getByAccessKey($access_key);
		if(is_null($estimate) || (!logged_user()->isMember() 
		&& logged_user()->getId() != $estimate->getClientId()) ) {
		
			set_flash_error(lang('e_3'));
			redirect('dashboard');
		
		}

		tpl_assign('estimate', $estimate);
		
	}
	
	public function create() {

		if(!logged_user()->isOwner()) die();
		only_ajax_request_allowed();

		$this->setLayout('modal');
		$this->setTemplate('estimates/_estimate_form');

		$estimate = new Estimate();
		tpl_assign('estimate', $estimate);

		$client_users = $this->Users->getByClients();
		tpl_assign('client_users', $client_users);

		$projects = $this->Projects->getAll();
		tpl_assign('projects', $projects);

		$subject = input_post_request('subject');
		tpl_assign("subject", $subject);

		$client_id = input_post_request('client_id');
		tpl_assign("client_id", $client_id);

		$project_id = input_post_request('project_id');
		tpl_assign("project_id", $project_id);
	
		$status = input_post_request('status', 1);
		tpl_assign("status", $status);

		$currency = input_post_request('currency');
		tpl_assign("currency", $currency);

		$due_date = input_post_request('due_date');
		tpl_assign("due_date", $due_date);

		$tax = input_post_request('tax');
		tpl_assign("tax", $tax);

		$tax_rate = input_post_request('tax_rate');
		tpl_assign("tax_rate", $tax_rate);

		$items = input_post_request('items', 0);
		$is_submited = input_post_request('submited') ==  'submited';
		
		if ($is_submited) {

			$this->form_validation->set_rules('subject', lang('c_126'), 'trim|required|max_length[100]');
			$this->form_validation->set_rules('client_id', lang('c_29'), 'required|greater_than[0]',  array('greater_than' => lang('c_129')));
					
			if ($this->form_validation->run() == FALSE) {
				$this->renderText(output_ajax_request(false, validation_errors()));
			} else {
		
				try{
					
					$this->db->trans_begin();

					$estimate->setClientId($client_id);
					$estimate->setProjectId($project_id);

					$estimate->setStatus($status == 1);

					$estimate->setSubject($subject);
					$estimate->setDueDate($due_date);

					$estimate->setTax($tax);
					$estimate->setTaxRate($tax_rate);

					$estimate->setCreatedById(logged_user()->getId());

					$estimate->save();
					
					$sub_total=0;
					$o_key = 0;
					
					for ($i=1;$i<=$items;$i++) {
					
						$delete = intval(input_post_request("delete_" . $i));
						if (!$delete) {

							$name = input_post_request("name_" . $i);
							$quantity = abs(input_post_request("quantity_" . $i));
							$amount = abs(input_post_request("amount_" . $i));

							if ($quantity > 0 && $amount > 0 && $name != "") {
								
								$estimate_item = new EstimateItem();

								$o_key++;
								$estimate_item->setOKey($o_key);
								
								$item_hash_id = sha1(uniqid(rand(), true).$estimate->getId().
								$quantity.$amount.$name.$o_key.time());
								
								$estimate_item->setUKey($item_hash_id);

								$estimate_item->setEstimateId($estimate->getId());
								$estimate_item->setQuantity($quantity);
								$estimate_item->setAmount($amount);
								$estimate_item->setDescription($name);
												
								$estimate_item->save();
												
								$sub_total += $amount*$quantity;
							
							}
							
						}
					}

					if ($tax_rate > 0) {
					
						$tax_amount = abs($sub_total/100*$tax_rate);
						$sub_total += $tax_amount;
					
					}					

					$estimate->setTotalAmount($sub_total);
					
					$estimate->setEstimateNo("EST-".str_pad($estimate->getId(), 6, '0', STR_PAD_LEFT));
					$estimate->setAccessKey(sha1(uniqid(rand(), true).$estimate->getId().time()));
					
					$estimate->save();

					$this->db->trans_commit();
					
					// Send notification ..
					$email_to_client = intval(input_post_request("email_to_client"));
					if($email_to_client) {
 
						$notify_subject = lang('c_127');
						$notify_message = $this->load->view("emails/estimate", 
						array("estimate_subject" => $estimate->getSubject(), "estimate_no" => $estimate->getEstimateNo(), 
						"estimate_link" => get_page_base_url($estimate->getObjectURL())), true);

						$notify_user_object = $this->Users->findById($client_id);
						if(isset($notify_user_object)) {
							try {
								$this->UserNotifications->create($notify_user_object, $notify_subject, $notify_message);
								send_mail_SMTP($notify_user_object->getEmail(), $notify_user_object->getName(), $notify_subject, $notify_message);
							} catch (Exception $e){}
						}
							
					}
					
					set_flash_success(sprintf(lang('c_128'), lang('c_130')));
		
				} catch (Exception $e) {

					$this->db->trans_rollback();
					set_flash_error(lang('e_1'));

				}

				$this->renderText(output_ajax_request(true));

			}

		}
			
	}

	public function edit($id) {

		if(!logged_user()->isOwner()) die();
		only_ajax_request_allowed();

		$this->setLayout('modal');
		$this->setTemplate('estimates/_estimate_form');

		$estimate = $this->Estimates->findById($id);
		if(is_null($estimate)) set_flash_error(lang('e_3'), true);
		
		tpl_assign('estimate', $estimate);

		$client_users = $this->Users->getByClients();
		tpl_assign('client_users', $client_users);

		$projects = $this->Projects->getAll();
		tpl_assign('projects', $projects);

		$subject = input_post_request('subject', $estimate->getSubject());
		tpl_assign("subject", $subject);

		$client_id = input_post_request('client_id', $estimate->getClientId());
		tpl_assign("client_id", $client_id);

		$project_id = input_post_request('project_id', $estimate->getProjectId());
		tpl_assign("project_id", $project_id);
	
		$status = input_post_request('status', (int) $estimate->getStatus());
		tpl_assign("status", $status);

		$due_date_timestamp = $estimate->getDueDate();
		$formatted_due_date = $due_date_timestamp ? format_date($due_date_timestamp, 'm/d/Y') : null;
		
		$due_date = input_post_request('due_date', $formatted_due_date);
		tpl_assign("due_date", $due_date);

		$tax = input_post_request('tax', $estimate->getTax());
		tpl_assign("tax", $tax);

		$tax_rate = input_post_request('tax_rate', $estimate->getTaxRate());
		tpl_assign("tax_rate", $tax_rate);

		$items = input_post_request('items', 0);
		$is_submited = input_post_request('submited') ==  'submited';

		if ($is_submited) {

			$this->form_validation->set_rules('subject', lang('c_126'), 'trim|required|max_length[100]');
			$this->form_validation->set_rules('client_id', lang('c_29'), 'required|greater_than[0]',  array('greater_than' => lang('c_129')));
					
			if ($this->form_validation->run() == FALSE) {
				$this->renderText(output_ajax_request(false, validation_errors()));
			} else {
		
				try{
					
					$this->db->trans_begin();

					$estimate->setClientId($client_id);
					$estimate->setProjectId($project_id);

					$estimate->setStatus($status == 1);

					$estimate->setSubject($subject);
					$estimate->setDueDate($due_date);

					$estimate->setTax($tax);
					$estimate->setTaxRate($tax_rate);
					
					$estimate->clearEstimateItems();

					$sub_total=0;
					$o_key = 0;
					
					for ($i=1;$i<=$items;$i++) {
					
						$delete = intval(input_post_request("delete_" . $i));
						if (!$delete) {

							$name = input_post_request("name_" . $i);
							$quantity = abs(input_post_request("quantity_" . $i));
							$amount = abs(input_post_request("amount_" . $i));

							if ($quantity > 0 && $amount > 0 && $name != "") {
								
								$estimate_item = new EstimateItem();

								$o_key++;
								$estimate_item->setOKey($o_key);
								
								$item_hash_id = sha1(uniqid(rand(), true).$estimate->getId().
								$quantity.$amount.$name.$o_key.time());
								
								$estimate_item->setUKey($item_hash_id);

								$estimate_item->setEstimateId($estimate->getId());
								$estimate_item->setQuantity($quantity);
								$estimate_item->setAmount($amount);
								$estimate_item->setDescription($name);
												
								$estimate_item->save();
												
								$sub_total += $amount*$quantity;
							
							}
							
						}
					}

					if ($tax_rate > 0) {
					
						$tax_amount = abs($sub_total/100*$tax_rate);
						$sub_total += $tax_amount;
					
					}					

					$estimate->setTotalAmount($sub_total);
					$estimate->save();

					$this->db->trans_commit();
					
					// Send notification ..
					$email_to_client = intval(input_post_request("email_to_client"));
					if($email_to_client) {
 
						$notify_subject = lang('c_127');
						$notify_message = $this->load->view("emails/estimate", 
						array("estimate_subject" => $estimate->getSubject(), "estimate_no" => $estimate->getEstimateNo(), 
						"estimate_link" => get_page_base_url($estimate->getObjectURL())), true);

						$notify_user_object = $this->Users->findById($client_id);
						if(isset($notify_user_object)) {
							try {
								$this->UserNotifications->create($notify_user_object, $notify_subject, $notify_message);
								send_mail_SMTP($notify_user_object->getEmail(), $notify_user_object->getName(), $notify_subject, $notify_message);
							} catch (Exception $e){}
						}
							
					}
					
					set_flash_success(sprintf(lang('c_57'), lang('c_130')));
		
				} catch (Exception $e) {

					$this->db->trans_rollback();
					set_flash_error(lang('e_1'));

				}

				$this->renderText(output_ajax_request(true));

			}
			
		} 
			
	}

	public function convert($id) {

		if(!logged_user()->isOwner()) redirect("dashboard");
		
		$estimate = $this->Estimates->findById($id);
		if(isset($estimate)) {

			try {
			
				$this->db->trans_begin();

				$invoice = new Invoice();
				
				$invoice->setClientId($estimate->getClientId());
				$invoice->setProjectId($estimate->getProjectId());
				$invoice->setSubject($estimate->getSubject());

				$invoice->setStatus($estimate->getStatus());
				$invoice->setDueDate($estimate->getDueDate());

				$invoice->setTax($estimate->getTax());
				$invoice->setTaxRate($estimate->getTaxRate());

				$invoice->setCreatedById($estimate->getCreatedById());
				$invoice->save();

				$sub_total=0;
				$o_key = 0;
				
				$estimate_items = $estimate->getItems();
				if(isset($estimate_items) && is_array($estimate_items) && count($estimate_items)) {

					foreach($estimate_items as $estimate_item) {
					
						$invoice_item = new InvoiceItem();
	
						$o_key++;
						$invoice_item->setOKey($o_key);
						
						$item_hash_id = sha1(uniqid(rand(), true).$invoice->getId().
						$estimate_item->getQuantity().$estimate_item->getAmount().
						$estimate_item->getDescription().$o_key.time());
						
						$invoice_item->setUKey($item_hash_id);
	
						$invoice_item->setInvoiceId($invoice->getId());
						$invoice_item->setQuantity($estimate_item->getQuantity());
						$invoice_item->setAmount($estimate_item->getAmount());
						$invoice_item->setDescription($estimate_item->getDescription());
										
						$invoice_item->save();
										
						$sub_total += $estimate_item->getAmount()*$estimate_item->getQuantity();
					
					}
				
				}

				if ($invoice->getTaxRate() > 0) {
				
					$tax_amount = abs($sub_total/100*$invoice->getTaxRate());
					$sub_total += $tax_amount;
				
				}					
								
				$invoice->setTotalAmount($sub_total);
				$invoice->setInvoiceNo("INV-".str_pad($invoice->getId(), 6, '0', STR_PAD_LEFT));
				$invoice->setAccessKey(sha1(uniqid(rand(), true).$invoice->getId().time()));
				
				$invoice->save();
				
				// finished ..
				$estimate->delete();

				$this->db->trans_commit();
				set_flash_success(sprintf(lang('c_131'), lang('c_130')));

			} catch(Exception $e) {

				$this->db->trans_rollback();
				set_flash_error(lang('e_1'));

			}
			
		}
				
		redirect("invoices");

	}
		
}
