<?php defined('BASEPATH') OR exit('No direct script access allowed');

class Timesheet extends Application_controller {
	
	function __construct() {
		
		parent::__construct();
		initial_data_contoller($this);

 		if(!logged_user()->isMember()) redirect('dashboard');
				
	}
	
	public function index() {
		
		$projects = logged_user()->getActiveProjects();
		tpl_assign('projects', $projects);
		
		$project_ids = get_objects_ids($projects);

		$project_id = input_get_request('project_id', 0);
		tpl_assign('project_id', $project_id);

		$selected_project_ids = ($project_id > 0 && in_array($project_id, $project_ids)) ? 
		array($project_id) : $project_ids;

		$timelogs = $this->ProjectTimelogs->getByProjects($selected_project_ids);
		tpl_assign('timelogs', $timelogs);
	
	}

	public function validate_date($date) {
		return $date != '' ? validate_date($date, 'Y-m-d H:i') : true;
	}

	public function compare_dates() {
	
		$start = strtotime($this->input->post('start_date'));
		$end = strtotime($this->input->post('end_date'));
	
		if($start > $end) {
			$this->form_validation->set_message('compare_dates',lang('c_191'));
			return false;
		}

	}

	public function create($request_project_id = null) {
			
		only_ajax_request_allowed();

		$this->setLayout('modal');
		$this->setTemplate('timesheet/_timelog_form');

		$project_timelog = new ProjectTimelog();
		tpl_assign('project_timelog', $project_timelog);

		if(logged_user()->isOwner() || logged_user()->isAdmin()) {

			$user_id = input_post_request('user_id', 0);
			tpl_assign("user_id", $user_id);

			$is_approved = input_post_request('is_approved', 1);
			tpl_assign("is_approved", $is_approved);

		} else {

			$is_approved = false;
			tpl_assign("is_approved", $is_approved);
		
			$user_id = logged_user()->getId();
			tpl_assign("user_id", $user_id);
		
		} 
				
		$memo = input_post_request('memo');
		tpl_assign("memo", $memo);

		$start_date = input_post_request('start_date');
		tpl_assign("start_date", $start_date);
		
		$end_date = input_post_request('end_date');
		tpl_assign("end_date", $end_date);

		$projects = logged_user()->getActiveProjects();
		tpl_assign('projects', $projects);

		$project_id = (int) input_post_request('project_id', $request_project_id);
		tpl_assign("project_id", $project_id);
	
		$is_submited = input_post_request('submited') ==  'submited';
		
		if ($is_submited) {

			$this->form_validation->set_rules('project_id', lang('c_23'), 'required|greater_than[0]',  array('greater_than' => lang('c_336')));
			
			if(logged_user()->isOwner() || logged_user()->isAdmin()) {
				$this->form_validation->set_rules('user_id', lang('c_28'), 'required|greater_than[0]',  array('greater_than' => lang('c_351')));
			}
			
			$this->form_validation->set_rules('start_date', lang('c_192'), 'required|callback_validate_date',  
			array('validate_date' => lang('c_352')));

			$this->form_validation->set_rules('end_date', lang('c_198'), 'required|callback_validate_date',  
			array('validate_date' => lang('c_353')));

			$this->form_validation->set_rules('end_date', lang('c_198'), 'trim|callback_compare_dates');

			$this->form_validation->set_rules('memo', lang('c_354'), 'trim|required|max_length[100]');
							
			if ($this->form_validation->run() == FALSE) {
				$this->renderText(output_ajax_request(false, validation_errors()));
			} else {
		
				try{
					
					$project_timelog->setMemo($memo);
					$project_timelog->setProjectId($project_id);
					$project_timelog->setMemberId($user_id);
					$project_timelog->setStartTime($start_date);
					$project_timelog->setEndTime($end_date);
					$project_timelog->setTotalHours(round((strtotime($end_date)-strtotime($start_date))/3600, 2));
					
					$project_timelog->setIsApproved($is_approved == 1);

					$project_timelog->setCreatedById(logged_user()->getId());
					$project_timelog->save();

					$this->ActivityLogs->create($project_timelog, lang('c_355'), 'add', true, true);
					
					set_flash_success(sprintf(lang('c_54'), lang('c_356')));
		
				} catch (Exception $e) {
					set_flash_error(lang('e_1'));
				}

				$this->renderText(output_ajax_request(true));

			}

		}
			
	}

	public function edit($id) {

        if(!(logged_user()->isOwner() || logged_user()->isAdmin())) die();

		only_ajax_request_allowed();

		$this->setLayout('modal');
		$this->setTemplate('timesheet/_timelog_form');
		
		$active_projects = logged_user()->getActiveProjects();
		$project_ids = get_objects_ids($active_projects);

		$project_timelog = $this->ProjectTimelogs->findById($id);

		if(is_null($project_timelog) || !in_array($project_timelog->getProjectId(), $project_ids) ) {
			set_flash_error(lang('e_3'), true);
		}
		
		tpl_assign('project_timelog', $project_timelog);
		tpl_assign('projects', $active_projects);

		$project_id = input_post_request('project_id');
		tpl_assign("project_id", $project_id);

		if(logged_user()->isOwner() || logged_user()->isAdmin()) {

			$user_id = input_post_request('user_id', $project_timelog->getMemberId());
			tpl_assign("user_id", $user_id);

		} else {
			
			if(logged_user()->getId() == $project_timelog->getMemberId()) {
				$user_id = $project_timelog->getMemberId();
				tpl_assign("user_id", $user_id);
			} else {
				set_flash_error(lang('e_3'), true);
			}		
	
		} 
				
		$memo = input_post_request('memo', $project_timelog->getMemo());
		tpl_assign("memo", $memo.$project_timelog->getStartDate());

		$start_date_timestamp = $project_timelog->getStartTime();
		$formatted_start_date = $start_date_timestamp ? format_date($start_date_timestamp, 'Y-m-d H:i') : null;
		
		$start_date = input_post_request('start_date', $formatted_start_date);
		tpl_assign("start_date", $start_date);

		$end_date_timestamp = $project_timelog->getEndTime();
		$formatted_end_date = $end_date_timestamp ? format_date($end_date_timestamp, 'Y-m-d H:i') : null;
		
		$end_date = input_post_request('end_date', $formatted_end_date);
		tpl_assign("end_date", $end_date);

		$project_id = (int) input_post_request('project_id', $project_timelog->getProjectId());
		tpl_assign("project_id", $project_id);

		$is_approved = input_post_request('is_approved', (int) $project_timelog->getIsApproved());
		tpl_assign("is_approved", $is_approved);
	
		$is_submited = input_post_request('submited') ==  'submited';

		if ($is_submited) {

			$this->form_validation->set_rules('user_id', lang('c_28'), 'required|greater_than[0]',  array('greater_than' => lang('c_351')));

			$this->form_validation->set_rules('start_date', lang('c_192'), 'required|callback_validate_date',  
			array('validate_date' => lang('c_352')));

			$this->form_validation->set_rules('end_date', lang('c_198'), 'required|callback_validate_date',  
			array('validate_date' => lang('c_353')));

			$this->form_validation->set_rules('end_date', lang('c_198'), 'trim|callback_compare_dates');

			$this->form_validation->set_rules('memo', lang('c_354'), 'trim|required|max_length[100]');
							
			if ($this->form_validation->run() == FALSE) {
				$this->renderText(output_ajax_request(false, validation_errors()));
			} else {
		
				try{
					
					$project_timelog->setMemo($memo);
					$project_timelog->setStartTime($start_date);
					$project_timelog->setEndTime($end_date);
					$project_timelog->setTotalHours(round((strtotime($end_date)-strtotime($start_date))/3600, 2));

					if(logged_user()->isOwner() || logged_user()->isAdmin()) {
						$project_timelog->setMemberId($user_id);
					}
					
					$project_timelog->setIsApproved($is_approved == 1);
					$project_timelog->save();

					$this->ActivityLogs->create($project_timelog, lang('c_357'), 'edit', true, true);
					
					set_flash_success(sprintf(lang('c_57'), lang('c_356')));
		
				} catch (Exception $e) {
					set_flash_error(lang('e_1'));
				}

				$this->renderText(output_ajax_request(true));

			}

		} 
			
	}
		
}
