<?php defined('BASEPATH') OR exit('No direct script access allowed');

/* ProjectFiles Model */

class ProjectFiles_model extends Application_model {

    function __construct() {
        parent::__construct('project_files', 'ProjectFile');
    }

	public function getByAccessKey($access_key) {
		return  $this->find(array('conditions' => array('`access_key` = ?', $access_key), 'one' => true));
	}

	public function getByParent(Application_object $parent_object, $include_private = false, $include_trashed = false) {

		return $this->find(array('conditions' => array('`is_trashed` <= ? AND `is_private` <= ? AND `parent_type` = ? AND `parent_id` = ?', 
		$include_trashed, $include_private, $parent_object->getModelName(), $parent_object->getId()), 'order' => '`created_at` DESC'));

	}

	public function getByProject(Project $project, $include_trashed = false, $include_private = false, $limit = null, $offset = null) {
		
		return $this->find(array('conditions' => array('`is_trashed` <= ? AND `is_private` <= ? AND `project_id` = ?', 
		$include_trashed, $include_private, $project->getId()), 
		'order' => '`created_at` DESC',
		'limit' => $limit,
        'offset' => $offset
		));
						
	}

	public function getByTrashed() {
		return $this->find(array('conditions' => array('`is_trashed` = ?', true), 'order' => '`file_name`'));
	}

	public function saveFile($attach_file, $can_download, Application_object $attach_to){

		if(is_array($attach_file) && $attach_file['name'] != "") {
			
			$file_name = basename($attach_file['name']);
			$file_extension = get_file_extension($file_name);
			$file_type = $attach_file['type'];
			$file_size = $attach_file['size'];
			$file_source = $attach_file['tmp_name'];
			
			$file_repository = new FileRepository();
			if($repo_id = $file_repository->updateFileContent($file_source)) {

				try{

					$project_file = new ProjectFile();
					
					if($attach_to instanceof Project) {
						$project_file->setProjectId($attach_to->getId());
					} else {

						$project_file->setProjectId($attach_to->getProjectId());
						$project_file->setParentType($attach_to->getModelName());
						$project_file->setParentId($attach_to->getId());

					}

					$project_file->setAccessKey(sha1(uniqid(rand(), true).$attach_to->getId().$file_source.time()));
					$project_file->setCanDownload($can_download);
					$project_file->setFileName($file_name);
					$project_file->setFileRepositoryId($repo_id);
					$project_file->setFileExtension($file_extension);
					$project_file->setFileTypeString($file_type);
					$project_file->setFileSize($file_size);
					
					$project_file->setCreatedById(logged_user()->getId());
					$project_file->save();
					
					return $project_file;
					
				} catch(Exception $e) {
								
					if(isset($repo_id)) {
						
						if(isset($file_repository) && $file_repository instanceof FileRepository) {
							$file_repository->deleteFile($repo_id);
						}
						
					}

					return false;
										
				}
								
			}
			
		}

		return true;
		
	}

}

/* ProjectFile Object */

class ProjectFile extends Application_object {

	function __construct() {
		parent::__construct('project_files');
	}

	public function getName() {
		return $this->getFileName();
	}

	public function moveToTrash() {
		
		$this->setIsTrashed(true);
		$this->save();
			
	}

	public function restoreFromTrash() {
		
		$this->setIsTrashed(false);
		$this->save();

	}

	function delete(){

		$file_repository = new FileRepository();
		$file_repository->deleteFile($this->getFileRepositoryId());

		return parent::delete();
			
	}

	/* Static URLs */

    function getShowFileURL() {
		return 'files/show/'.$this->getId();
	}

    function getHideFileURL() {
		return 'files/hide/'.$this->getId();
	}

    function getObjectURL() {
		return 'files/download/'.$this->getAccessKey();
	}

}